bl_info = {
    "name": "ZipMerge",
    "blender": (3, 60, 0),
    "description": "Merge at least 3 verts between your mesh islands, select the vert you want to start merging from and run ZipWeld under the merge menu.",
    "location": "Edit Mode > Mesh > Merge",
    "version": (1, 21),
    "category": "Object",
    "author": "Nick Hodgson",
}

import bpy
import bmesh

class OBJECT_OT_zip_merge(bpy.types.Operator):
    bl_idname = "object.zip_merge"
    bl_label = "ZipMerge"
    bl_options = {'REGISTER', 'UNDO'}
    
    iterations: bpy.props.IntProperty(
        name="Iterations",
        default=1,
        min=1,
        description="Number of iterations"
    )
    check_common_elements: bpy.props.BoolProperty(
        name="Stop At Corners",
        description="Enable check for more than 3 common elements",
        default=False
    )
    
    def growMergeVertex(self, context):
        obj = context.object
        if obj and obj.type == 'MESH' and obj.mode == 'EDIT':
            mesh = obj.data
            bm = bmesh.from_edit_mesh(mesh)
            bm.verts.ensure_lookup_table()
            
            start_vertex = [v.index for v in bm.verts if v.select]
            
            bpy.ops.mesh.select_more()
            
            expanded_verts = [v.index for v in bm.verts if v.select]
            
            bpy.ops.mesh.select_all(action='DESELECT')
            
            boundary_verts = [v.index for v in bm.verts if v.is_boundary]
            
            common_elements = list(set(expanded_verts) & set(boundary_verts))
            
            if self.check_common_elements and len(common_elements) > 3:
                return False  # Signal to stop iterations
            
            set_start_vertex = set(start_vertex)
            set_common_elements = set(common_elements)
            
            merge_verts = list(set_common_elements - set_start_vertex)
            
            for v_idx in merge_verts:
                bm.verts[v_idx].select = True
                
            bpy.ops.mesh.merge(type='CENTER')
            
            bmesh.update_edit_mesh(mesh)
        return True

    def execute(self, context):
        num_iterations = self.iterations
        
        for i in range(num_iterations):
            if not self.growMergeVertex(context):
                self.report({'INFO'}, f"Stopped after {i+1} iterations due to reaching a corner junction.")
                break
        
        return {'FINISHED'}

def menu_func(self, context):
    self.layout.operator(OBJECT_OT_zip_merge.bl_idname)

def register():
    bpy.utils.register_class(OBJECT_OT_zip_merge)
    bpy.types.VIEW3D_MT_edit_mesh_merge.append(menu_func)

def unregister():
    bpy.utils.unregister_class(OBJECT_OT_zip_merge)
    bpy.types.VIEW3D_MT_edit_mesh_merge.remove(menu_func)

if __name__ == "__main__":
    register()